%option reentrant
%option bison-bridge

/* get rid of non-portable calls: isatty, fileno */
%option never-interactive
%{
/*  gentrex - generic tree expressions; tokenizer
    Copyright (C) 2015  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/gentrex
    Contact the author (email and IRC): http://igor2.repo.hu/contact.html
*/

#include <stdio.h>
#include <ctype.h>
#include "execute.h"

#define YY_DECL int gtx_lex(YYSTYPE * yylval_param ,yyscan_t yyscanner, gentrex_state_t *gtx)

#include "gentrex.gram.h"
#include "lex_helper.h"

static char *read_pattern();

%}

ALNUM [A-Za-z0-9]
ID    [A-Za-z0-9_]
DIGIT [0-9]
DQUOTE (\\\"|[^\"])

%%

["]{DQUOTE}*["]     { yylval->str = strcloneq(yytext, '"'); return yylval->str == NULL ? -1 : STRING; }
[~]                 { yylval->str = read_pattern(gtx, yyg, yyscanner); return MATCH_STRING; }
{DIGIT}*            { gtx->sval = yytext; yylval->i = atoi(yytext); return INTEGER; }
{ID}*               { yylval->str = strclone(yytext); return STRING; }
[|][|]              { return OP_OR; }
[&][&]              { return OP_AND; }
[=][=]              { return OP_EQ; }
[*.<>=[\]()/{}+!:^-] { return *yytext; }

[ \t\r\n]           { }

.                   { fprintf(stderr, "lex invalid char: %c\n", *yytext); }

%%

/* has to be in this file for yyscan_t and input() */
static char *read_pattern(gentrex_state_t *gtx, struct yyguts_t *yyg, yyscan_t yyscanner)
{
	int sep, i;
	char *s = NULL;
	int used = 0, alloced = 0;

	while(isspace(sep = input(yyscanner))) ;
	if ((sep < 32) || (sep > 127))
		return NULL;

	if (gtx->match_c_op != NULL)
		free(gtx->match_c_op);

	gtx->match_c_op = malloc(2);
	gtx->match_c_op[0] = sep;
	gtx->match_c_op[1] = '\0';

	for(;;) {
		i = input(yyscanner);
		if (i == EOF)
			return NULL;

		/* grow before checking the sep so we always have room for a \0 */
		if (used >= alloced) {
			alloced += 256;
			s = realloc(s, alloced);
		}
		if (i == sep)
			break;
		s[used] = i;
		used++;
	}
	s[used] = '\0';
	return s;
}


